#!/bin/bash
#
# Script name: configure-mounts.sh
#
# Script location: /usr/local/bin (or elsewhere in root's PATH)
#
# Detect partitions and types, configure them in fstab, make mountpoints and mount them, rw access for all users
#
# Optionally, write a new fstab
#
# For Debian Squeeze. Wheezy/sid at May 2011, later maybe or not. Can be used in a live session
#
# Non-essential removable drives should be unplugged before use
#
# Maybe or not works for hfs, nobody here with hfs to test
#
# Something similar has been done before but probably pre-etch, pre-uuid and blkid,
# no info available to provide appropriate credits except Kano (Kanotix)
#
# Copyright: David Hare <exe-linux@fastfishwebsolutions.com> Jan 2012 
# License: GPL3
# Traducción (espanol): Alexis Puente Montiel, < alexispm_stellaluna@yahoo.es >
#
# Experimental use only at user risk, not fully tested
#
# Will include usb only if option "usb" is given when calling script (e.g.: configure-mounts.sh usb)
#

TEXTDOMAIN=configure-mounts.sh
TEXTDOMAINDIR=/usr/share/locale/

TITLE="configure-mounts.sh" # Script title (could be different)

###### EDITABLE CONFIGS ###########################################

# LINUX filesystem mount options
LINUXOPTS="rw,user,suid,dev,exec,auto,async"
# defaults : LINUXOPTS="rw,suid,dev,exec,auto,nouser,async"

# FAT filesystem mount options
FATOPTS="rw,user,auto,fmask=0111,dmask=0000"
#FATOPTS=rw,users,fmask=0111,dmask=0000 #default

# NTFS-3G (will be autodetected) mount options NOTE: mountable only by root
NTFS3GOPTS="auto,user,exec,rw,umask=000"

# NTFS filesystem (without -3g) mount options
NTFSOPTS="ro,user,auto,noexec,umask=000"

# HFS filesystem mount options (not tested)
HFSOPTS="user,file_umask=0111,dir_umask=0000"

# Mount what is detected when script has finished?
MOUNTNOW="yes"

###################################################################

# detect system language
SCRIPTLANG=$(env | egrep 'LC|LANG' |grep -o "es_")

if [ "$SCRIPTLANG" = "es_" ]; then 

	#  if exists, call alt language subscript (if exists) and exit
	if [ -f "/usr/local/bin/exegnu-scripts-es/configure-mounts-es.sh" ]; then
	/usr/local/bin/exegnu-scripts-es/configure-mounts-es.sh
	# stop this script
	exit 1
	fi
fi

###########################################################################################################################

# check for root
	if [[ $UID != 0 ]]; then
	echo $"$TITLE must be run as root"
	exit 1
	fi

FSTABDIR=/etc
FSTAB=/etc/fstab
MEDIA=/media

###########################################################################################################################

echo ""
echo $"$TITLE will detect partitions, configure them in fstab, make mountpoints and mount them"
echo ""
echo $"All non-essential removable disks MUST be unmounted and unplugged" # because we don't want them in fstab
echo ""
echo $"Please type yes to continue (or any key to exit now) then <ENTER>"

read REPLY
if ! [ "$REPLY" = $"yes" ]; then
exit 1
fi

echo ""
echo "--------------------------------------------------------"
echo ""

# first, backup current fstab

# leave fstab.original alone if exists (might be useful for persistent)
if ! [ -f /etc/fstab.original ]; then
cp -a $FSTAB /etc/fstab.original
FSTABBACKUP=/etc/fstab.original
else
# write or overwrite fstab.last
cp -a $FSTAB /etc/fstab.last
FSTABBACKUP=/etc/fstab.last
fi


### check if is a live session

LIVESESSIONCHECK=$(cat /proc/cmdline|tr [A-Z] [a-z]|grep -o "boot=live"|grep -o "live")

############ get root partition info ######

# (for live session only) usually aufs but might not be
UNIONFILESYSTEM=$(cat /proc/cmdline|grep -o union=.*|sed 's: .*::'|sed 's:union=::')

ROOTPARTENTRY=$(cat $FSTAB| sed 's/\t/ /'g| sed 's:/dev/disk/by-label/::'g| sed 's:/dev/disk/by-uuid/::'g| sed 's:UUID=::'g| sed 's:LABEL=::'g| sed 's:"::'g   |sed 's:#.*::'g| grep " / "|awk '{print $1}')

ROOTPART=$(/sbin/blkid|grep $ROOTPARTENTRY|awk '{print $1}'| sed 's/://' )
ROOTPARTUUID=$(/sbin/blkid|grep $ROOTPARTENTRY|grep -o "UUID=.*"|sed 's: .*::'|sed 's:UUID=::'|sed 's:"::'g )

if [ "$ROOTPART" = "" ] || [ "$ROOTPART" = "$UNIONFILESYSTEM" ]; then # grep won't work for blank patterns
ROOTPART="no_rootpart"
ROOTPARTUUID="no_rootpart_uuid"
fi

ROOTPARTDEV=$(echo $ROOTPART|sed 's:[0-9]::'g)

#### get boot partition info (if exists) 

BOOTPARTENTRY=$(cat $FSTAB| sed 's/\t/ /'g| sed 's:/dev/disk/by-label/::'g| sed 's:/dev/disk/by-uuid/::'g| sed 's:UUID=::'g| sed 's:LABEL=::'g| sed 's:"::'g   |sed 's:#.*::'g| grep " /boot "|awk '{print $1}')

if [ "$BOOTPARTENTRY" = "" ]; then
BOOTPARTENTRY="no_bootpart_entry"
BOOTPART="no_bootpart"
BOOTPARTUUID="no_bootpart_uuid"

else
BOOTPART=$(/sbin/blkid|grep $BOOTPARTENTRY |awk '{print $1}'| sed 's/://')
BOOTPARTUUID=$( /sbin/blkid|grep $BOOTPART|grep -o "UUID=.*"|sed 's: .*::'|sed 's:UUID=::'|sed 's:"::'g )

fi

#### get home partition info (if exists)

HOMEPARTENTRY=$(cat $FSTAB| sed 's/\t/ /'g| sed 's:/dev/disk/by-label/::'g| sed 's:/dev/disk/by-uuid/::'g| sed 's:UUID=::'g| sed 's:LABEL=::'g| sed 's:"::'g|sed 's:#.*::'g| grep " /home "|awk '{print $1}')

if [ "$HOMEPARTENTRY" = "" ]; then
HOMEPARTENTRY="no_homepart_entry"
HOMEPART="no_homepart"
HOMEPARTUUID="no_homepart_uuid"
else
HOMEPART=$(/sbin/blkid|grep $HOMEPARTENTRY |awk '{print $1}'| sed 's/://')
HOMEPARTUUID=$( /sbin/blkid|grep $HOMEPART|grep -o "UUID=.*"|sed 's: .*::'|sed 's:UUID=::'|sed 's:"::'g )
fi

### get swap partition info (if exists)

SWAPPARTENTRY=$(cat $FSTAB| sed 's/\t/ /'g| sed 's:/dev/disk/by-label/::'g| sed 's:/dev/disk/by-uuid/::'g| sed 's:UUID=::'g| sed 's:"::'g|sed 's:#.*::'g| grep -m 1 " swap "|awk '{print $1}')

if [ "$SWAPPARTENTRY" = "" ]; then # there is none
SWAPPARTENTRY="no_swappart_entry"
SWAPPART="no_swappart"
SWAPPARTUUID="no_swappart_uuid"
fi

if ! [ "$SWAPPARTENTRY" = "" ]; then # it exists 
SWAPPART=$(/sbin/blkid|grep $SWAPPARTENTRY |awk '{print $1}'| sed 's/://')
	if [ "$SWAPPART" = "" ]; then # it is not a device, must be a swapfile
	SWAPPART="no_swappart"
	SWAPPARTUUID="no_swappart_uuid"
	
	else # it is a real device
	SWAPPARTUUID=$(/sbin/blkid|grep $SWAPPART|grep -o 'UUID=.*'|sed 's: .*::'|sed 's:UUID=::'|sed 's:"::'g)
	fi
fi

### FUTURE ###
#maybe something like this for an installer but need to take out the interactive bits and the echos
# maybe with an 'option'
#LIVESESSIONCHECK=$(cat /proc/cmdline | grep -i "boot=live")
#if [ "$LIVESESSIONCHECK" = "" ]; then
# check if this is called by an installer
#ROOTPART=$INSTALLATIONMOUNTPOINT
#ROOTPARTUUID=$(blkid -o udev $ROOTPART | grep UUID= | cut -f2 | sed -e 's/ID_FS_//' | sed -e 's/UUID=//')
#fi

### OPTION TO CLEAR AND REBUILD FSTAB (using uuid) STARTS HERE #########################################################

echo "" # space out the lines in terminal for readability
echo $" WARNING: Answering 'yes' to this will overwrite your current fstab"
echo $" Custom settings may be deleted ! "
echo $" Type n (or any key) if unsure "
echo $"Do you need to clear the current fstab?  (yes/n/other)"
echo ""

read CLEARFSTAB

if [ "$CLEARFSTAB" = "yes" ] ; then

#clear fstab of all except headers, / (rootfs), cdrom, swap, boot, home, var, proc, aufs and tmpfs lines 

egrep -i "information|dump|$ROOTPART|$ROOTPARTUUID|iso9660|swap|boot|home|mapper|var|proc|aufs|tmpfs" $FSTAB  >/tmp/fstab-temp && cp -a /tmp/fstab-temp $FSTAB && rm -f /tmp/fstab-temp

# delete all comments
sed -i 's/#.*//'g $FSTAB

# delete all blank lines
sed -i '/^ *$/d' $FSTAB

# replace headers, they just got deleted
sed -i '1i # <file system> <mount point> <type> <options> <dump> <pass>' $FSTAB
sed -i '1i # /etc/fstab: static file system information.' $FSTAB

# insert new blank lines
sed -i G  $FSTAB

# convert $ROOTPART to $ROOTPARTUUID (but not if is a live session)

	if [ "$LIVESESSIONCHECK" = "" ]; then

	# convert /dev entries to uuid (will do nothing if LABEL or swapfile)
	sed -i "s:$ROOTPART:UUID=$ROOTPARTUUID:" $FSTAB
	sed -i "s:$SWAPPART:UUID=$SWAPPARTUUID:" $FSTAB
	sed -i "s:$BOOTPART:UUID=$BOOTPARTUUID:" $FSTAB
	sed -i "s:$HOMEPART:UUID=$HOMEPARTUUID:" $FSTAB

	# insert new readability comments only if is UUID
	sed -i "/UUID=$ROOTPARTUUID/i # $ROOTPART (rootfs)" $FSTAB
	sed -i "/UUID=$SWAPPARTUUID/i # $SWAPPART (swap)" $FSTAB
	sed -i "/UUID=$BOOTPARTUUID/i # $BOOTPART (/boot)" $FSTAB
	sed -i "/UUID=$HOMEPARTUUID/i # $HOMEPART (/home)" $FSTAB

	fi
fi
echo ""

#### APPENDS START HERE ##############################################################################################

# add a blank line first
#echo "" >>/etc/fstab

# we want to exclude live mounts... (live-media could be on /live/image or /live/findiso)
# and write a value for it even if it does not exist (else grep errors later)
# will output probably more than one line
LIVEMOUNT=$(cat /proc/mounts|grep "/live/"|grep "/dev/sd[a-z]"|awk '{print $1}')

if [ "$LIVEMOUNT" = "" ]; then
LIVEMOUNT="unconfigured_livemount"
fi

# live-rw mount ?
LIVERW=$(cat /proc/mounts|grep "/live-rw"|grep "/dev/"|awk '{print $1}')

if [ "$LIVERW" = "" ]; then
LIVERW="no_liverw"
fi

# identify removables and exclude the first device found UNLESS it is a usb "proper installation"

REMOVABLE1=$(dmesg|grep removable|grep "\[sd[a-z]\]"|grep -o "\[sd[a-z]\]"|sed 's:\[::' |sed  's:\]::'|tr '\n' ' '|awk '{print $1}')

# a isohybrid will show up as a removable so see if a second usb is plugged
REMOVABLE2=$(dmesg|grep removable|grep "\[sd[a-z]\]"|grep -o "\[sd[a-z]\]"|sed 's:\[::' |sed  's:\]::'|tr '\n' ' '|awk '{print $2}')

if [ "$ROOTPARTDEV" = "/dev/${REMOVABLE1}" ]; then
REMOVABLE1=include_usb
fi

if [ "${REMOVABLE1}" = "" ]; then
REMOVABLE1=no_removable1
fi

if [ "${REMOVABLE2}" = "" ]; then
REMOVABLE2=no_removable2
fi

### identify LUKS parts to exclude ######################

CRYPTO1=$(/sbin/blkid|grep /dev/sd|grep crypto|sed 's@:@@'|grep -o "/dev/sd[a-z][0-9]"|tr '\n' ' '|awk '{print $1}')

CRYPTO2=$(/sbin/blkid|grep /dev/sd|grep crypto|sed 's@:@@'|grep -o "/dev/sd[a-z][0-9]"|tr '\n' ' '|awk '{print $2}')

CRYPTO3=$(/sbin/blkid|grep /dev/sd|grep crypto|sed 's@:@@'|grep -o "/dev/sd[a-z][0-9]"|tr '\n' ' '|awk '{print $3}')

CRYPTO4=$(/sbin/blkid|grep /dev/sd|grep crypto|sed 's@:@@'|grep -o "/dev/sd[a-z][0-9]"|tr '\n' ' '|awk '{print $4}')


if [ "$CRYPTO1" = "" ]; then
CRYPTO1="no_crypto1"
else 
echo "Detected LUKS volume $CRYPTO1"
echo
fi

if [ "$CRYPTO2" = "" ]; then
CRYPTO2="no_crypto2"
else
echo "Detected LUKS volume $CRYPTO2"
fi


if [ "$CRYPTO3" = "" ]; then
CRYPTO3="no_crypto3"
else
echo "Detected LUKS volume $CRYPTO3"
fi

if [ "$CRYPTO4" = "" ]; then
CRYPTO4="no_crypto4"
else
echo "Detected LUKS volume $CRYPTO4"
fi

### SORT LINUX PARTITIONS #######################################################################################################

# grep out swap root boot home and live-mount partitions # divert fdisk warnings to /dev/null

# this will not work if $ROOTPART $BOOTPART $HOMEPART $LIVEMOUNT $LIVERW are blank !
LINUXPARTS=$(/sbin/fdisk -l  2>/dev/null|grep '/dev/sd'| grep -i 'linux'| grep -v 'swap'| grep -v "$ROOTPART"| grep -v "$LIVEMOUNT"| grep -v "$LIVERW"|grep -v "$BOOTPART"| grep -v "$HOMEPART"| grep -v "$REMOVABLE1"| grep -v "$REMOVABLE2"| grep -v "$CRYPTO1"| grep -v "$CRYPTO2"| grep -v "$CRYPTO3"| grep -v "$CRYPTO4"| sed 's:/dev/::'g| awk '{print $1}')

for PART in $LINUXPARTS; do
	# see if it's NOT in fstab
	if [[ ! `grep -w '$PART' $FSTAB` ]]; then

		# get it's UUID
		if [[ ! `grep $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $FSTAB` ]]; then

			# make a mountpoint  unless already exists
			if ! [ -d $MEDIA/$PART ]; then
				echo $"Creating mountpoint $MEDIA/$PART..."
				mkdir "$MEDIA/$PART"
			else
				echo $"Mountpoint $MEDIA/$PART already exists!"
			fi

		# add a blank line
		echo "" >> $FSTAB
		
		# insert a readability comment above
		echo "# /dev/$PART (added by script 'configure-mounts.sh')" >> $FSTAB

		# write the entry to fstab
		echo  "$(/sbin/blkid -o udev /dev/$PART $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $MEDIA/$PART auto $LINUXOPTS 0 0" >> $FSTAB

        	echo $"Added /dev/$PART $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') mountpoint: $MEDIA/$PART"
        	echo ""
        	DETECTED='yes'
		fi
	else
        	echo $"/dev/$PART is already configured!"
		echo ""
	fi
done

### SORT FAT PARTITIONS ########################################################################################################

FATPARTS=$(/sbin/fdisk -l  2>/dev/null | grep -v "$LIVEMOUNT"| grep -v "$LIVERW"| grep -i 'fat'| grep -v "exFAT"| grep -v "$REMOVABLE1"|grep -v "$REMOVABLE2"| sed 's:/dev/::'g|awk '{print $1}')

for PART in $FATPARTS; do
	if [[ ! `grep -w '$PART' $FSTAB` ]]; then

		if [[ ! `grep $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $FSTAB` ]]; then

			if ! [ -d $MEDIA/$PART ]; then
			echo "Creating mountpoint $MEDIA/$PART..."
			mkdir "$MEDIA/$PART"
			else
			echo "Mountpoint $MEDIA/$PART already exists!"
			fi

		echo "" >> $FSTAB
		echo "# /dev/$PART (added by script 'configure-mounts.sh')" >> $FSTAB

		echo  "$(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $MEDIA/$PART vfat $FATOPTS 0 0" >> $FSTAB

		echo $"Added /dev/$PART $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') mountpoint: $MEDIA/$PART"
		echo ""
		DETECTED=yes

		fi

	else
		echo $"/dev/$PART is already configured!"
		echo ""
	fi
done

### SORT NTFS PARTITIONS #######################################################################################################

NTFSPARTS=`/sbin/fdisk -l  2>/dev/null | grep -i 'ntfs' | grep -v "$REMOVABLE1"| grep -v "$REMOVABLE2"| awk -F '/| ' '{print $3}'`

if [ "$(which ntfs-3g)" = "" ];then
NTFSTYPE="ntfs"
else
	NTFSTYPE="ntfs-3g"
	NTFSOPTS="$NTFS3GOPTS"
fi

for PART in $NTFSPARTS; do
	if [[ ! `grep -w '$PART' $FSTAB` ]]; then

		if [[ ! `grep $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $FSTAB` ]]; then

			if ! [ -d $MEDIA/$PART ]; then
			echo "Creating mountpoint $MEDIA/$PART..."
			mkdir "$MEDIA/$PART"
			else
			echo "Mountpoint $MEDIA/$PART already exists!"
			fi


		echo "" >> $FSTAB
		echo "# /dev/$PART (added by script 'configure-mounts.sh')" >> $FSTAB
		echo  "$(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $MEDIA/$PART $NTFSTYPE $NTFSOPTS 0 0" >> $FSTAB

		echo $"Added /dev/$PART $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') mountpoint: $MEDIA/$PART"
		echo ""
		DETECTED="yes"

		fi
	else
	echo $"/dev/$PART is already configured!"
	echo ""
	fi
done

### SORT HFS PARTITIONS #######################################################################################################

# hfs (mac) partitions # totally untested! might work

HFSPARTS=`/sbin/fdisk -l  2>/dev/null | grep -i 'af' | awk -F '/| ' '{print $3}'`

for PART in $HFSPARTS; do
	if [[ ! `grep -w '$PART' $FSTAB` ]]; then

		if [[ ! `grep $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $FSTAB` ]]; then

			if ! [ -d $MEDIA/$PART ]; then
			echo "Creating mountpoint $MEDIA/$PART..."
			mkdir "$MEDIA/$PART"
			else
			echo "Mountpoint $MEDIA/$PART already exists!"
			fi

		echo ""
		echo "# /dev/$PART (added by script 'configure-mounts.sh')" >> $FSTAB

		echo  "$(/sbin/blkid -o udev /dev/$PART $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') $MEDIA/$PART hfsplus rw,$HFSOPTS 0 0" >> $FSTAB
		echo "" >> $FSTAB
		echo $"Added /dev/$PART $(/sbin/blkid -o udev /dev/$PART | grep "UUID=" | cut -f2 | sed -e 's/ID_FS_//') mountpoint: $MEDIA/$PART"

		DETECTED='yes'
		fi

	else
		echo $"/dev/$PART is already configured!"	
	fi
done

### MOUNT WHAT WAS DETECTED & save old fstab ################################################################################

if ! [ "$DETECTED" = "yes" ]; then

	echo $"No new partitions have been detected"
else
		if [ "$MOUNTNOW" = "yes" ]; then
		echo ""
		echo $"Mounting detected partitions..."
		mount -a
		else
		echo $"Detected partitions configured but not mounted..."
		fi
	echo ""
	echo $"All done. The original fstab was saved to $FSTABBACKUP"
fi

exit 1
